<?php

namespace Drupal\phonepay_payment\Methods;

use Drupal\phonepay_payment\PhonePeBaseClass;

/**
 * Class to porvide methods to perform PhonePe API Calls.
 */
class StandardCheckout extends PhonePeBaseClass {

  /**
   * Readonly property.
   * 
   * @var array
   */
  private $transaction;

  /**
   * Readonly property.
   * 
   * @var array
   */
  private $transactionResponse;

  /**
   * Readonly property.
   *
   */
  const TRANSACTION_STATUS_SUCCESS = "PAYMENT_SUCCESS";

  /**
   * Create a new transaction for the user.
   *
   * @param int $amountInPaisa
   *   Amount.
   * @param string $mobile
   *   Mobile Number.
   * @param string $merchentTransactionID
   *   Transaction ID.
   * @param string $redirectUrl
   *   Redirect URL.
   * @param string $callbackUrl
   *   CallBack URL.
   * @param string $redirectMode
   *   RedirectMode.
   *
   * @return StandardCheckout
   *   Return Checkout.
   */
  public function createTransaction($amountInPaisa, $mobile, $merchentTransactionID, $redirectUrl = NULL, $callbackUrl = NULL, $redirectMode = "REDIRECT") {
    // Check if the amount is valid.
    if (!is_numeric($amountInPaisa)) {
      throw new \Exception("Amount should be numeric");
    }

    // Check if the mobile number is valid.
    if (!is_numeric($mobile)) {
      throw new \Exception("Mobile number should be numeric");
    }

    // According to the api, the redirect mode should be REDIRECT or POST.
    if ($redirectMode != "REDIRECT" && $redirectMode != "POST") {
      throw new \Exception("Invalid redirect mode, should be REDIRECT or POST");
    }

    if (!$redirectUrl) {
      $redirectUrl = $this->config['REDIRECT_URL'];
    }
    if (!$callbackUrl) {
      $callbackUrl = $this->config['CALLBACK_URL'];
    }

    $payload = [
      "merchantId" => $this->config['MERCHANT_ID'],
      "merchantTransactionId" => $merchentTransactionID,
      "merchantUserId" => $this->config['MERCHANT_USER_ID'],
      "amount" => $amountInPaisa,
      "redirectUrl" => $redirectUrl,
      "redirectMode" => $redirectMode,
      "callbackUrl" => $callbackUrl,
      "mobileNumber" => $mobile,
      "paymentInstrument" => [
        "type" => "PAY_PAGE",
      ],
    ];

    $response = $this->executeRequest("POST", "/pg/v1/pay", $payload);
    $this->transaction = $response;
    return $this;
  }

  /**
   * If the transaction is created.
   *
   * @return array
   *   Return the full transaction response from the server.
   */
  public function getCreatedTransaction() {
    if (!$this->transaction) {
      throw new \Exception("Transaction not created");
    }

    return $this->transaction;
  }

  /**
   * If the transaction is created.
   *
   * @return string
   *   Return the transaction URL from the transaction response.
   */
  public function getTransactionUrl() {
    if (!$this->transaction) {
      throw new \Exception("Transaction not created");
    }

    return $this->transaction['data']['instrumentResponse']['redirectInfo']['url'];
  }

  /**
   * Verifies the response from the server using phonepe's algorithm.
   *
   * @param string $payload
   *   Payload.
   * @param string $xVerify
   *   xVerify.
   *
   * @return bool
   *   Return Verify Response.
   */
  public function verifyResponse($payload, $xVerify) {
    return parent::verifyResponse($payload, $xVerify);
  }

  /**
   * Verifies the response from the server using phonepe's algorithm.
   *
   * @param string $payload
   *   Payload.
   * @param string $xVerify
   *   xVerify.
   *
   * @return array|bool
   *   Returns the decoded response.
   */
  public function verifyAndGetResponse($payload, $xVerify) {
    if ($this->verifyResponse($payload, $xVerify)) {
      return json_decode(base64_decode($payload), TRUE);
    }
    return FALSE;
  }

  /**
   * Verifies the response from the server directly using the global variables.
   *
   * @return array
   *   Return Payload.
   */
  public function verifyPaymentFromCallback() {
    // Check if the request method is not POST.
    if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
      throw new \Exception("Invalid request method");
    }

    // Check if X-VERIFY is set.
    if (!isset($_SERVER['HTTP_X_VERIFY'])) {
      throw new \Exception("X-VERIFY not set");
    }
    $xVerify = $_SERVER['HTTP_X_VERIFY'];

    // Get Content.
    $content = trim(file_get_contents("php://input"));
    $payload = json_decode($content, TRUE);
    $base64Payload = $payload['response'];

    // Validate X-VERIFY.
    if (!$this->verifyResponse($base64Payload, $xVerify)) {
      throw new \Exception("Invalid X-VERIFY");
    }

    // Decode Payload.
    $payload = json_decode(base64_decode($base64Payload), TRUE);

    // Store transaction response.
    $this->transactionResponse = $payload;

    return $payload;
  }

  /**
   * Verifies the response from the server.
   *
   * @return array
   *   Return Response.
   */
  public function getTransactionResponse() {
    if (!$this->transactionResponse) {
      $this->verifyPaymentFromCallback();
    }

    return $this->transactionResponse;
  }

  /**
   * Verifies the response from the server.
   *
   * @return string
   *   Return Transaction ID.
   */
  public function getTransactionId() {
    if (!$this->transactionResponse) {
      $this->verifyPaymentFromCallback();
    }

    return $this->transactionResponse['data']['transactionId'];
  }

  /**
   * Verifies the response from the server.
   *
   * @return string
   *   Return Response Code.
   */
  public function getTransactionStatus() {
    if (!$this->transactionResponse) {
      $this->verifyPaymentFromCallback();
    }

    return $this->transactionResponse['code'];
  }

  /**
   * Verifies the response from the server.
   *
   * @return bool
   *   Return Status.
   */
  public function isTransactionSuccess() {
    if ($this->getTransactionStatus() == self::TRANSACTION_STATUS_SUCCESS) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Verifies the response from the server directly.
   *
   * @return int
   *   Return Amount.
   */
  public function getTransactionAmount() {
    if (!$this->transactionResponse) {
      $this->verifyPaymentFromCallback();
    }

    return $this->transactionResponse['data']['amount'];
  }

  /**
   * Verifies the response from the server.
   *
   * @return array
   *   Return Instrument.
   */
  public function getTransactionPaymentInstrument() {
    if (!$this->transactionResponse) {
      $this->verifyPaymentFromCallback();
    }

    return $this->transactionResponse['data']['paymentInstrument'];
  }

  /**
   * Verifies the response from the server.
   *
   * @return string
   *   Return instrument type.
   */
  public function getTransactionPaymentInstrumentType() {
    if (!$this->transactionResponse) {
      $this->verifyPaymentFromCallback();
    }

    return $this->transactionResponse['data']['paymentInstrument']['type'];
  }

  /**
   * Get the transaction response from the server using the transaction ID.
   *
   * @param string $transactionId
   *   Transaction ID.
   *
   * @return array
   *   Return Response.
   */
  public function getTransactionResponseByTransactionId($transactionId) {
    $response = $this->executeRequest(
      "GET", "/pg/v1/status/" . $this->config['MERCHANT_ID'] . "/" . $transactionId, NULL, [
        'X-MERCHANT-ID' => $this->config['MERCHANT_ID'],
      ]
    );
    return $response;
  }

  /**
   * Get the transaction status from the server using the transaction ID.
   *
   * @param string $transactionId
   *   Transaction ID.
   *
   * @return string
   *   Return response code.
   */
  public function getTransactionStatusByTransactionId($transactionId) {
    $response = $this->getTransactionResponseByTransactionId($transactionId);
    return $response['code'];
  }

  /**
   * Get if transaction is success or not from server using transaction ID.
   *
   * @param string $transactionId
   *   Transaction ID.
   *
   * @return bool
   *   Return transaction status.
   */
  public function isTransactionSuccessByTransactionId($transactionId) {
    if ($this->getTransactionStatusByTransactionId($transactionId) == self::TRANSACTION_STATUS_SUCCESS) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Get the transaction amount from the server using the transaction ID.
   *
   * @param string $transactionId
   *   Transaction ID.
   *
   * @return int
   *   Return amount.
   */
  public function getTransactionAmountByTransactionId($transactionId) {
    $response = $this->getTransactionResponseByTransactionId($transactionId);
    return $response['data']['amount'];
  }

  /**
   * Get transaction payment instrument from server using transaction ID.
   *
   * @param string $transactionId
   *   Transaction ID.
   *
   * @return array
   *   Return Payment Intrusment.
   */
  public function getTransactionPaymentInstrumentByTransactionId($transactionId) {
    $response = $this->getTransactionResponseByTransactionId($transactionId);
    return $response['data']['paymentInstrument'];
  }

  /**
   * Get transaction payment instrument type from server using transaction ID.
   *
   * @param string $transactionId
   *   Transaction ID.
   *
   * @return string
   *   Return the payment type.
   */
  public function getTransactionPaymentInstrumentTypeByTransactionId($transactionId) {
    $response = $this->getTransactionResponseByTransactionId($transactionId);
    return $response['data']['paymentInstrument']['type'];
  }

}
