<?php

namespace Drupal\phonepay_payment;

use Drupal\phonepay_payment\PhonePe;
use GuzzleHttp\Client;

/**
 * Class to porvide base methods.
 */
class PhonePeBaseClass {

  /**
   * Readonly property.
   */
  protected $config;

  /**
   * Readonly property.
   */
  protected $client;

  /**
   * {@inheritdoc}
   */
  public function __construct() {
    $this->client = new Client();
    $this->config = PhonePe::getInstance()->getConfig();
  }

  /**
   * Calculate the X-VERIFY header using the phonepe's algorithm.
   *
   * @param string $payload
   *   Payload.
   * @param string $path
   *   Path.
   * @param string $saltKey
   *   SaltKey.
   * @param string $saltIndex
   *   SaltIndex.
   *
   * @return string
   *   Return headers.
   */
  protected function calculateHeaders($payload, $path, $saltKey, $saltIndex) {
    $xVerify = hash('sha256', $payload . $path . $saltKey) . "###" . $saltIndex;

    return $xVerify;
  }

  /**
   * Executes the request and returns the response.
   *
   * @param string $method
   *   Method.
   * @param string $path
   *   Path.
   * @param array|null $payload
   *   Payload.
   * @param array $headers
   *   Headers.
   *
   * @return array
   *   Return Response.
   */
  protected function executeRequest($method, $path, $payload = NULL, $headers = []) {
    if ($payload) {
      // Making Original Payload.
      $base64Payload = base64_encode(json_encode($payload));
      $requestBody = json_encode(['request' => $base64Payload]);

      // Calculate X-VERIFY.
      $xVerify = $this->calculateHeaders($base64Payload, $path, $this->config['SALT_KEY'], $this->config['SALT_INDEX']);
    }
    else {
      $requestBody = NULL;
      $xVerify = $this->calculateHeaders("", $path, $this->config['SALT_KEY'], $this->config['SALT_INDEX']);
    }

    // Set the headers for PhonePe's Algorithm.
    $headers['Content-Type'] = 'application/json';
    $headers['X-VERIFY'] = $xVerify;

    // Define the options.
    $options = [
      'headers' => $headers,
      'body' => $requestBody,
    ];

    $response = $this->client->request($method, $this->config['HOST'] . $path, $options);
    return json_decode($response->getBody(), TRUE);
  }

  /**
   * Verifies the response from the server using phonepe's algorithm.
   *
   * @param string $payload
   *   Payload.
   * @param string $xVerify
   *   xVerify.
   *
   * @return bool
   *   Return Status.
   */
  protected function verifyResponse($payload, $xVerify) {
    $xVerify = explode("###", $xVerify);
    $xVerify = $xVerify[0];
    $xVerify = $xVerify . "###" . $this->config['SALT_INDEX'];

    $xVerifyCalculated = $this->calculateHeaders($payload, "", $this->config['SALT_KEY'], $this->config['SALT_INDEX']);

    if ($xVerify == $xVerifyCalculated) {
      return TRUE;
    }

    return FALSE;
  }

}
